#!/usr/local/bin/ruby -Ke
#
# Ruriko -- Ruby Remote Information Keeper Online --
#

def cgi?
  ENV.include?('QUERY_STRING')
end
Dir.chdir(File.dirname($0)) unless cgi?

$:[0, 0] = [ './usr', './sys' ]

require 'resolv-replace'
require 'kconv'
require 'html'
require 'setting'
require 'message'
require 'utility'
require 'category'
require 'site'
require 'sitefile'
require 'timezone'
require 'plugin'
require 'backup'

class CGIFailed < Exception; end
class CGISucceed < Exception; end

module Ruriko
  NAME = 'Ruriko'
  VERSION = '0.1.19'
  AUTHOR_URL = 'http://www.nekome.net/'
  USER_AGENT = "#{NAME}/#{VERSION}"
end

module Setting
  LOCAL_INFO_PATH = './data/local_info'
  LOCAL_INFO_INDEX_PATH = './data/local_info_index'
  BACKUP_DIR_PATH = './backup/'
  BACKUP_LIST_PATH ='./backup/list'
end


# ޥɥ饤
class ParseError < Exception; end

def parse_arg(args)
  raise ParseError if args.empty?

  until args.empty?
    arg = args.shift;
    case arg
    when '--verbose'
#      STDOUT.sync = true
      $verbose = true

    when '--observe'
      Thread.start do
        loop do
          Utility.log(Thread.list.size.to_s, Thread.main.inspect)
          sleep(1)
        end
      end

    when '--init'
      Utility.touch(Setting::LOCAL_INFO_PATH, true)
      Utility.touch(Setting::LOCAL_INFO_INDEX_PATH, true)
      Utility.touch(Setting::BACKUP_DIR_PATH, true)
      Utility.touch(Setting::BACKUP_LIST_PATH, true)
      Plugin.init
      
    when '--import'
      sites = Utility.import(args.shift)
      SiteFile.new(Setting::LOCAL_INFO_INDEX_PATH).merge(sites)

    when '--describe'
      sites = SiteFile.new(Setting::LOCAL_INFO_PATH).sites
      sites.each do |site| 
        site.describe(STDOUT)
        STDOUT.puts
      end

    when '--strip'
      file = SiteFile.new(Setting::LOCAL_INFO_PATH)
      file.strip

    when '--generate'
      sites = SiteFile.new(Setting::LOCAL_INFO_PATH).sites
      sites.sort!{|a, b| (b.valid_modified || 0) <=> (a.valid_modified || 0)}
      Plugin.generators.each do |generator|
	generator.generate(sites)
      end
      Backup.backup(Setting::LOCAL_INFO_INDEX_PATH)
      Backup.backup(Setting::LOCAL_INFO_PATH)
      
    when '--update'
      require 'timeout'
      Utility.log('begin update')
      file = SiteFile.new(Setting::LOCAL_INFO_PATH)
      file.conform(SiteFile.new(Setting::LOCAL_INFO_INDEX_PATH).sites)
      file.update
      Utility.log('end update')
      
    else 
      raise ParseError
    end
  end

rescue ParseError
  print <<-"END"
Usage: #{File.basename($0)} [options]
Options: [default in brackets after descriptions]
Configuration:
  --init    initialize data files
END
end


if cgi?
  CGI.new('html4').interface
else
  parse_arg(ARGV)
end



