require 'md5'

module Backup
  class FileInfo
    attr_reader :filename, :md5sum, :size, :ctime
    def initialize(filename, md5sum, size)
      @filename = filename
      @md5sum = md5sum
      @size = size
      @ctime = Time.now.to_i
    end
  end

  def backup(filename)
    Utility.exclusive_open(Setting::BACKUP_LIST_PATH, 'r+') do |listfile|
      list = []
      begin
        list = Marshal.load(listfile.read)
      rescue
      end

      data = Utility.exclusive_open(filename){|file| file.read}
      md5sum = MD5.new(data).digest
      new_filename = Setting::BACKUP_DIR_PATH + 
        File.basename(filename) + Time.now.strftime('.%Y%m%d%H%M%S')
      compressor = Plugin.compressors.find{|plugin|
        plugin.extension == Setting::BACKUP_FILE_EXTENSION
      }
      if compressor
        new_filename += '.' + compressor.extension
        data = compressor.compress(data)
      end

      new_info = FileInfo.new(new_filename, md5sum, data.size)
      unless list.find{|info| info.md5sum == new_info.md5sum}
        File.open(new_filename, 'w') do |file| 
          file.write(data)
          file.chmod(0666)
        end
        list << new_info
      end
    
     if Setting::BACKUP_LIMIT_TIME
        expire_date = Time.now.to_i - Setting::BACKUP_LIMIT_TIME
        list.reject! do |info| info.ctime < expire_date end
      end

      if Setting::BACKUP_LIMIT_SIZE
        sum = 0
        count = list.size
        list.reverse_each do |info|
          sum += info.size
          break if sum > Setting::BACKUP_LIMIT_SIZE
          count -= 1
        end
        list[0, count] = []
      end

      if Setting::BACKUP_LIMIT_NUM &&
          list.size > Setting::BACKUP_LIMIT_NUM
        list[0, list.size - Setting::BACKUP_LIMIT_NUM] = []
      end

      expires = Dir.glob(Setting::BACKUP_DIR_PATH + '*')
      expires -= list.collect{|info| info.filename}
      expires -= [ Setting::BACKUP_LIST_PATH ]

      expires.each do |filename|
        File.unlink(filename)
      end

      listfile.rewind
      listfile.truncate(0)
      listfile.write(Marshal.dump(list))
    end     
  end

  module_function :backup
end

