# CGI extention
#  see manual.html for detail.
#


require 'cgi'

STDIN.reopen('/dev/null') unless ENV.include?('REQUEST_METHOD')

class CGI
  BODY_OPTIONS = {
    'bgcolor' => 'white',
    'text' => 'black'
  }

  module Name
    KCODE = 'kcode'
    ACTION = 'action'
    URL = 'url'
    NEW_URL = 'new_url'
    TARGET_URL = 'target_url'
    TITLE = 'title'
    AUTHOR = 'author'
    CATEGORY = 'category'
    COMMENT = 'comment'
    PASSWORD = 'password'
    NEW_PASSWORD = 'new_password'
  end

  module Action
    ADD_SITE = 'add_site'
    DELETE_SITE = 'delete_site'
    MODIFY_SITE_REQUEST = 'modify_site_request'
    MODIFY_SITE = 'modify_site'
  end

  KCODE_MAGIC_STRING = 'ǯ'

  def a_cookie(name)
   cookies.include?(name) ? cookies[name].value[0] : nil
  end

  def a_param(name)
    params.include?(name) ? params[name][0] : nil
  end


  def jis_out(title, &block)
    cookies = self.cookies
    if include?(Name::URL) && include?(Name::PASSWORD)
      cookies = {}
      cookies['url'] = 
        CGI::Cookie.new('name' => 'url',
                        'value' => [ a_param(Name::URL) ])
      cookies['password'] =
        CGI::Cookie.new('name' => 'password',
                        'value' => [ a_param(Name::PASSWORD) ])
    end
    out('cookie' => cookies){ jis_html(title, &block) }
  end

  def jis_html(title, &block) 
    Kconv.tojis html {
      head {
        meta('http-equiv' => 'Content-Type',
             'content' => 'text/html; charset=iso-2022-jp') +
          title { title }
      } +
      body(BODY_OPTIONS) { yield }
    }
  end


  def footer
    hr + a('href' => Setting::HOME_URL) { Message::RETURN_TO_HOME }
  end

  def add_site
    h1 { Message::ADD_NEW_SITE } +
    form {
      hidden(Name::KCODE, KCODE_MAGIC_STRING) +
      hidden(Name::ACTION, Action::ADD_SITE) +
      strong{Message::URL} + br + 
      text_field(Name::URL, 'http://') + br +
      strong{Message::TARGET_URL} + br + 
      text_field(Name::TARGET_URL) + br +
      strong{Message::TITLE} + br + 
      text_field(Name::TITLE) + br +
      strong{Message::AUTHOR} + br + 
      text_field(Name::AUTHOR) + br +
      strong{Message::COMMENT} + br + 
      text_field(Name::COMMENT) + br +
      if Category.exist?
        strong{Message::CATEGORY} + br +
        Category.collect{|category|
          checkbox(Name::CATEGORY, category.name) + category.label
        }.join(' ') + br
      else
        ''
      end +
      if Setting::ALLOW_ONLY_ANONYMOUS_ADD
        hidden(Name::PASSWORD, '') 
      else
        strong{Message::PASSWORD} + br +
        password_field(Name::PASSWORD, a_cookie(Name::PASSWORD)) + br 
      end +
      submit(Message::SUBMIT) + 
      reset(Message::RESET)
    }
  end

  def delete_site
    h1 { Message::DELETE_SITE } +
    form {
      hidden(Name::KCODE, KCODE_MAGIC_STRING) + 
      hidden(Name::ACTION, Action::DELETE_SITE)+
      strong{Message::URL} + br +
      text_field(Name::URL, a_cookie(Name::URL) || 'http://') + br +
      strong{Message::PASSWORD} + br +
      password_field(Name::PASSWORD, a_cookie(Name::PASSWORD)) + br +
      submit(Message::SUBMIT) + 
      reset(Message::RESET)
    }
  end

  def modify_site_request
    h1 { Message::MODIFY_SITE } +
    form {
      hidden(Name::KCODE, KCODE_MAGIC_STRING) + 
      hidden(Name::ACTION, Action::MODIFY_SITE_REQUEST) +
      strong{Message::URL} + br +
      text_field(Name::URL, a_cookie(Name::URL) || 'http://') + br +
      strong{Message::PASSWORD} + br +
      password_field(Name::PASSWORD, a_cookie(Name::PASSWORD)) + br +
      submit(Message::SUBMIT) + 
      reset(Message::RESET)
    }
  end

  def modify_site(site, password)
    h1 { Message::MODIFY_SITE } +
    form {
      hidden(Name::KCODE, KCODE_MAGIC_STRING) +
      hidden(Name::ACTION, Action::MODIFY_SITE) +
      hidden(Name::URL, site.url) +
      strong{Message::URL} + br + 
      text_field(Name::NEW_URL, site.url) + br +
      strong{Message::TARGET_URL} + br + 
      text_field(Name::TARGET_URL, site.target_url) + br +
      strong{Message::TITLE} + br + 
      text_field(Name::TITLE, site.title) + br +
      strong{Message::AUTHOR} + br + 
      text_field(Name::AUTHOR, site.author) + br +
      strong{Message::COMMENT} + br + 
      text_field(Name::COMMENT, site.comment) + br +
      strong{Message::CATEGORY} + br +
      if Category.exist? 
        Category.collect{|category|
          checkbox(Name::CATEGORY, category.name,
                   site.category && site.category.include?(category.name)) +
          category.label
        }.join(' ') + br
      else
        ''
      end +
      strong{Message::PASSWORD} + br +
      password_field(Name::NEW_PASSWORD,
                password.auth(Setting::ADMIN_PASSWORD) ? '' : password) +
      hidden(Name::PASSWORD, password) + br +
      submit(Message::SUBMIT) + 
      reset(Message::RESET)
    }
  end    

  def toeuc(str)
    Kconv.kconv(str, Kconv::EUC, @kcode)
  end

  def interface
    if include?(Name::URL)
      a_param(Name::URL).sub!(/rrkdmy=\d{8,8}&/, '')
      a_param(Name::URL).sub!(/\?\d{8,8}$/, '')
    end

    @kcode = 
      if include?(Name::KCODE)
        Kconv.guess(a_param(Name::KCODE))
      else
        Kconv::AUTO
      end

    case a_param(Name::ACTION)

    when Action::ADD_SITE
      file = SiteFile.new(Setting::LOCAL_INFO_INDEX_PATH)
      file.add(a_param(Name::URL),
               a_param(Name::PASSWORD),
               a_param(Name::TARGET_URL),
               toeuc(a_param(Name::TITLE)),
               toeuc(a_param(Name::AUTHOR)),
               toeuc(a_param(Name::COMMENT)),
               params[Name::CATEGORY])
      
    when Action::DELETE_SITE
      file = SiteFile.new(Setting::LOCAL_INFO_INDEX_PATH)
      file.delete(a_param(Name::URL),
                  a_param(Name::PASSWORD))
      
    when Action::MODIFY_SITE
      file = SiteFile.new(Setting::LOCAL_INFO_INDEX_PATH)
      file.modify(a_param(Name::URL),
                  a_param(Name::PASSWORD),
                  a_param(Name::NEW_URL),
                  a_param(Name::TARGET_URL),
                  toeuc(a_param(Name::TITLE)),
                  toeuc(a_param(Name::AUTHOR)),
                  toeuc(a_param(Name::COMMENT)),
                  params[Name::CATEGORY],
                  a_param(Name::NEW_PASSWORD))
      
    when Action::MODIFY_SITE_REQUEST
      file = SiteFile.new(Setting::LOCAL_INFO_INDEX_PATH)
      site = file.find(a_param(Name::URL),
                       a_param(Name::PASSWORD))
      jis_out(Setting::TITLE) {
        modify_site(site, a_param(Name::PASSWORD)) + footer
      }

    else
      jis_out(Setting::TITLE) {
        add_site + delete_site + modify_site_request + footer
      }
    end

  rescue CGISucceed => e
    jis_out(Setting::TITLE + ':' + Message::SUCCEED) {
      strong{CGI.escapeHTML(e.message)} + br +
      form {
        submit(Message::SUBMIT)
      } + 
      footer
    }

  rescue CGIFailed => e
    jis_out(Setting::TITLE + ':' + Message::FAILED) {
      h1 { Message::FAILED } +
      strong{CGI.escapeHTML(e.message)} + br +
      form {
        submit(Message::SUBMIT)
      } +
      footer
    }

  rescue Exception => e
    jis_out(Setting::TITLE + ':' + Message::INTERNAL_ERROR) {
      h1 { Message::INTERNAL_ERROR } +
      strong{CGI.escapeHTML(e.message)} + br +
      e.backtrace.collect{|l| CGI.escapeHTML(l)}.join(br) +
      form {
        submit(Message::SUBMIT)
      } +
      footer
    }

  end
end

