class OptHandler
  class UndefinedOption < StandardError; end

  class Option
    attr_accessor :long_name, :short_name, :range, :default, :description, :proc
  end

  def initialize
    @options = {}
    @values = {'' => []}
    @help = []
  end

  def add(name, short_name = nil, hash = {})
    if @options.include?(name)
      raise RuntimeError, "option duplicated '#{name}."
    end

    option = @options[name] = Option.new
    option.long_name = name
    option.short_name = short_name
    option.range = hash['range'] || 0
    option.default = hash['default']
    option.description = hash['description']
    option.proc = proc if block_given?
    @values[name] = option.default if option.default
    @help << option if option.description
  end

  def delete(name)
    @help.delete(@options.delete(name))
    @values.delete(name)
  end

  def add_description(*args)
    @help.concat(args)
  end

  def help
    max_opt_len = @help.collect{|a| a.kind_of?(Option) ? a.long_name.size : 0}.max
    @help.collect{|a|
      if a.kind_of?(Option)
        "  --#{a.long_name}#{' ' * (max_opt_len - a.long_name.size)}  #{a.description}\n"
      else
        "#{a}\n"
      end
    }.join
  end

  def include?(name)
    @values.include?(name)
  end

  def [](name)
    @values[name]
  end

  def parse(args)
    args.each do |arg|
      case arg
      when /\A--(.+)/
      when /\A-(.+)/
      else
      end
    end
  end
end
