# ץ饰

module Plugin
  PLUGIN_PATH = [ './usr/plugin', './sys/plugin' ]
  TRANSLATOR_PATH = 'translator'
  GENERATOR_PATH = 'generator'
  CLIPPER_PATH = 'clipper'
  PARSER_PATH = 'parser'
  COMPRESSOR_PATH = 'compressor'

  def load(path, *required_methods)
    required_methods << :active
    plugin = Object.new
    plugin.instance_eval(File.open(path){|file| file.read})
    required_methods.each do |method|
      unless plugin.respond_to?(method)
        STDERR.print "#{path}: undefined method '#{method}'\n"
        return nil
      end
    end
    plugin.active ? plugin : nil

  rescue Exception => e
    STDERR.print "#{path}: #{e.message}\n"
    nil
  end

  def load_dir(path, *required_methods)
    plugins = {}
    files = PLUGIN_PATH.collect{|p| Dir.glob("#{p}/#{path}/*.rb")}.flatten
    files.each do |filename|
      key = File.basename(filename)
      next if plugins.include? key
      plugin = load(filename, *required_methods)
      plugins[key] = plugin if plugin
    end
    plugins.values
  end

  def translators
    Thread.critical = true
    @translators ||= load_dir(TRANSLATOR_PATH, :import, :export)
  ensure
    Thread.critical = false
  end

  def generators
    Thread.critical = true
    @generators ||= load_dir(GENERATOR_PATH, :generate)
  ensure
    Thread.critical = false
  end

  def clippers
    Thread.critical = true
    @clippers ||= load_dir(CLIPPER_PATH, :match, :clip)
  ensure
    Thread.critical = false
  end

  def parsers
    Thread.critical = true
    @parsers ||= load_dir(PARSER_PATH, :match, :parse)
  ensure
    Thread.critical = false
  end

  def compressors
    Thread.critical = true
    @compressors ||= load_dir(COMPRESSOR_PATH, :compress, :expand)
  ensure
    Thread.critical = false
  end

  def init
    plugins = generators + translators + clippers + parsers + compressors
    plugins.each do |plugin|
      plugin.init if plugin.respond_to?(:init)
    end
  end

  module_function :load, :load_dir, :init,
    :generators, :translators, :clippers, :parsers, :compressors
end  
