def active
  true
end

CRLF = "\xd\xa"

VERSION = "2.2"

module Field
  URL = 'Url'
  USER_AGENT = 'User-Agent'
  HINA_VERSION = 'Hina-Version'
  VIRTUAL = 'Virtual'
  TITLE = 'Title'
  CONTENT_TYPE = 'Content-Type'
  DATE = 'Date'
  AUTHOR_NAME = 'Author-Name'
  EXPIRES = 'Expires'
  EXPIRE = 'Expire'
  LAST_MODIFIED = 'Last-Modified'
  LAST_MODIFIED_DETECTED = 'Last-Modified-Detected'
  SERVER = 'Server'
  AUTHORIZED = 'Authorized'
  AUTHORIZED_URL = 'Authorized-Url'
  METHOD = 'Method'
  KEYWORD = 'Keyword'
  IMAGE_WIDTH = 'Image-Width'
  IMAGE_HEIGHT = 'Image-Height'
  EXPERIMENTAL_FIELD = 'Experimental-Field'
end

def extension
  'di'
end

def import(input)
  sites = []
  field = {}
  kcode = Kconv::EUC
  header_done = false

  input.each do |line|
    line.strip!
    
    matched = /(.*?):(.*)/.match(line)
    if matched
      field_name = matched[1].split('-').collect{|s| s.capitalize}.join('-')
      field_value = matched[2].strip
      field[field_name] = field_value
      next
    end

    next unless line.empty?

    begin
      unless header_done
        header_done = true
        if field.include?(Field::CONTENT_TYPE)
          matched = /charset=(.*?);?/.match(field[Field::CONTENT_TYPE])
          if matched
            case matched[1]
            when /iso-2022-jp/i
              kcode = Kconv::JIS
            when /shift-jis/i
              kcode = Kconv::SJIS
            end
          end
        end        
      end

      next unless field.include?(Field::URL)

      site = Site.new
      site.url = field[Field::URL]
      site.title = field[Field::TITLE] || ''
      site.author = field[Field::AUTHOR_NAME] || ''
      site.target_url = field[Field::VIRTUAL] || site.url
      if field.include?(Field::LAST_MODIFIED)
        site.last_modified = 
          Time.rfc822time(field[Field::LAST_MODIFIED]).to_i
      end
      if field.include?(Field::LAST_MODIFIED_DETECTED)
        site.last_detected =
          Time.rfc822time(field[Field::LAST_MODIFIED_DETECTED]).to_i
        end
      if field.include?(Field::EXPIRES)
        site.expires =
          Time.rfc822time(field[Field::EXPIRES]).to_i
      end
      
      sites << site
    ensure
      field.clear
    end
  end
  sites
end

def export(output, sites)
#  server = `uname -n`.strip
#  server = nil if server.empty?
  date = Time.now.utc.rfc822fmt

  output << "HINA/#{VERSION}#{CRLF}" 
  output << "#{Field::USER_AGENT}: #{Ruriko::USER_AGENT}#{CRLF}"
  output << "#{Field::DATE}: #{date}#{CRLF}"
  output << CRLF

  sites.each do |site|
    output << "#{Field::URL}: #{site.url}#{CRLF}"
#    output << "#{Field::HINA_VERSION}: #{VERSION}#{CRLF}"

    if site.url.uniq_url != site.target_url.uniq_url
      output << "#{Field::VIRTUAL}: #{site.target_url}#{CRLF}"
    end

    output << "#{Field::DATE}: #{date}#{CRLF}"

    output << "#{Field::TITLE}: #{site.title}#{CRLF}"

    output << "#{Field::AUTHOR_NAME}: #{site.author}#{CRLF}"

    if site.expires
      output << "#{Field::EXPIRES}: #{Time.at(site.expires).utc.rfc822fmt}#{CRLF}"
      output << "#{Field::EXPIRE}: #{Time.at(site.expires).utc.rfc822fmt}#{CRLF}"
    end

    if site.last_modified
      output << "#{Field::LAST_MODIFIED}: #{Time.at(site.last_modified).utc.rfc822fmt}#{CRLF}"
    end

    if site.last_detected
      output << "#{Field::LAST_MODIFIED_DETECTED}: #{Time.at(site.last_detected).utc.rfc822fmt}#{CRLF}"
    end

#    if server
#      output << "#{Field::SERVER}: #{server}#{CRLF}"
#    end

    output << "#{Field::AUTHORIZED}: #{Ruriko::USER_AGENT}#{CRLF}"

    output << "#{Field::AUTHORIZED_URL}: #{Setting::SCRIPT_URL}#{CRLF}"

    output << CRLF
  end
end

