class SiteFile
  def initialize(filename)
    @filename = filename
  end

  def dump
    @file.rewind
    Marshal.dump(@sites, @file)
    @file.truncate(@file.pos)
    @dumped = true
  end

  def load
    @sites = []
    begin
      @sites = Marshal.load(@file)
    rescue Exception
    end
  end

  def hoge
    begin
    rescue
    ensure
    end
  end

  def open
    @dumped = false
    Utility.exclusive_open(@filename, 'r+') do |file|
      @file = file
      yield
    end
  ensure
#    Backup.backup(Setting::LOCAL_INFO_INDEX_PATH) if @dumped
#    raise CGIFailed if @dumped
#    Utility.exclusive_open(@filename) do end
#    hoge
  end

  def auth(url, password)
    site = @sites.find{|site| site.url.uniq_url == url.uniq_url}

    unless site
      raise CGIFailed, Message::SITE_NOT_FOUND
    end
    
    if password.auth(Setting::ADMIN_PASSWORD)
      return site
    end

    if Setting::ALLOW_ONLY_ADMIN_ACCESS || ! password.auth(site.password)
      raise CGIFailed, Message::INVALID_PASSWORD
    end

    site
  end

  def add(url, password, target_url, title, author, comment, category)
    target_url = url if target_url.empty?

    if (Setting::IGNORE_URL.detect{|u| url.uniq_url.include?(u)} ||
        Setting::IGNORE_URL.detect{|u| target_url.uniq_url.include?(u)})
      raise CGIFailed, Message::SITE_IGNORED
    end

    raise CGIFailed, Message::INPUT_URL if url.empty?
    raise CGIFailed, Message::INPUT_TITLE if title.empty?
    raise CGIFailed, Message::INPUT_AUTHOR if author.empty?
    raise CGIFailed, Message::INVALID_URL unless Utility.parse_url(url)
    unless Utility.parse_url(target_url)
      raise CGIFailed, Message::INVALID_TARGET_URL 
    end

    if Setting::ALLOW_ONLY_ADMIN_ACCESS &&
	! password.auth(Setting::ADMIN_PASSWORD)
	raise CGIFailed, Message::PERMISSION_DENIED
      end

    if ! Setting::CONFIRM_NULL_PASSWORD_AS_ADMIN_WHEN_ADD &&
	password.empty?
      raise CGIFailed, Message::INPUT_PASSWORD
    end

    new_site = Site.new
    new_site.url = url
    new_site.target_url = target_url
    new_site.title = title
    new_site.author = author
    new_site.comment = comment
    new_site.category = category

    if Setting::CONFIRM_NULL_PASSWORD_AS_ADMIN_WHEN_ADD &&
        password.empty?
      new_site.password = Setting::ADMIN_PASSWORD
    else
      new_site.password = password.crypt(Utility.salt)
    end

    open do
      load

      if @sites.find{|site| site.url.uniq_url == new_site.url.uniq_url}
	raise CGIFailed, Message::SITE_ALREADY_EXIST
      end

      @sites << new_site

      dump

      raise CGISucceed, Message::SITE_ADD_SUCCEED
    end

    raise CGIFailed, Message::SITE_ADD_FAILED
  end

  def delete(url, password)
    open do
      load

      site = auth(url, password)
      @sites.delete(site)

      dump

      raise CGISucceed, Message::SITE_DELETE_SUCCEED
    end

    raise CGIFailed, Message::SITE_DELETE_FAILED
  end

  def modify(url, password, new_url, target_url,
             title, author, comment, category, new_password)
    target_url = url if target_url.empty?
    
    raise CGIFailed, Message::INPUT_TITLE if title.empty?
    raise CGIFailed, Message::INPUT_AUTHOR if author.empty?
    raise CGIFailed, Message::INVALID_URL unless Utility.parse_url(url)
    raise CGIFailed, Message::INVALID_URL unless Utility.parse_url(new_url)
    unless Utility.parse_url(target_url)
      raise CGIFailed, Message::INVALID_TARGET_URL 
    end

    if new_password.empty? && ! password.auth(Setting::ADMIN_PASSWORD) 
      raise CGIFailed, Message::INPUT_PASSWORD
    end

    open do
      load

      site = auth(url, password)

      unless url.uniq_url == new_url.uniq_url
        if @sites.find{|s| s.url.uniq_url == new_url.uniq_url}
          raise CGIFailed, Message::SITE_ALREADY_EXIST
        end
        url.replace(new_url)
      end

      site.url = url
      site.target_url = target_url
      site.title = title # Kconv.kconv(title, Kconv::EUC, $cgi_kcode)
      site.author = author # Kconv.kconv(author, Kconv::EUC, $cgi_kcode)
      site.comment = comment
      site.category = category
      if ! new_password.empty? || ! password.auth(Setting::ADMIN_PASSWORD)
	site.password = new_password.crypt(Utility.salt)
      end

      dump

      raise CGISucceed, Message::SITE_MODIFY_SUCCEED
    end

    raise CGIFailed, Message::SITE_MODIFY_FAILED
  end

  def find(url, password)
    open do
      load

      return auth(url, password)
    end

    nil
  end

  def update
    open do
      load

      @sites.parallelize(Setting::MAX_HTTP_CONNECTIONS) do |site|
        begin
          site.update
        rescue TimeoutError
          site.log(e.message) # , *e.backtrace)
        rescue Exception => e
          site.log(e.message) # , *e.backtrace)
#          puts site.url, e.message, *e.backtrace
        end
      end

      dump

      return @sites
    end

    nil
  end	  

  def strip
    open do
      load

      @sites.each do |site|
	site.last_modified = nil 
	site.last_visited = nil
	site.md5digest = nil
      end

      dump
    end
  end

  def sites
    open do
      load
      return @sites
    end
  end


  def merge(op_sites)
    open do 
      load

      op_sites.reject!{|op_site|
        @sites.find{|site| 
          site.url.uniq_url == op_site.url.uniq_url
        }
      }

      @sites.concat(op_sites)

      dump
    end
  end

  def conform(op_sites)
    open do
      load

      @sites = op_sites.collect do |op_site|
	site = @sites.find{|site| site.url.uniq_url == op_site.url.uniq_url}
	if site
	  site.url = op_site.url
	  site.target_url = op_site.target_url
	  site.title = op_site.title
	  site.author = op_site.author
          site.category = op_site.category
          site.comment = op_site.comment
	  site
	else
	  op_site
	end
      end

      dump
    end
  end
end



