require 'parsedate'

class Time
  def rfc822fmt(local = false)
    if self.utc?
      self.utc.strftime("%a, #{self.mday} %b %Y %X GMT")
    else
      ret = self.strftime("%a, #{self.mday} %b %Y %X ")
      gmt = Time.at(self.to_i)
      gmt.gmtime
      offset = self.to_i - Time.local(*gmt.to_a[0,6].reverse).to_i
      ret + sprintf('%+.2d%.2d', *(offset / 60).divmod(60))
    end
  end
end

class << Time
  def lag
    now = Time.now
    Time.utc( *now.to_a[0,6].reverse ).to_i - now.to_i
  end

  def rfc822time(str)
    date = ParseDate.parsedate(str)
    Time.gm(*date[0, 6]).to_i - Timezone.tz2sec(date[6])
  end
end

class String
  def auth(password)
    self.crypt(password) == password
  end

  def uniq_url
    CGI.unescape(self.sub(/(\/\/.*?\/)/){$1.downcase})
  end

  def external_filter(cmd)
    require 'open3'
    pipe = Open3.popen3(cmd)
    Thread.start do
      s = 0
      while s < size 
        s += pipe[0].write(self[s, 1024])
      end
      pipe[0].close
    end
    pipe[1].read
  end
end

module Enumerable
  def parallelize(max_thread_count = nil)
    thread_count = 0
    main_thread = Thread.current
    each do |a|
      Thread.start(a) do |a|
        begin
          yield(a)
        ensure
          Thread.critical = true
          thread_count -= 1
          main_thread.wakeup
          Thread.critical = false
        end
      end
      Thread.critical = true
      thread_count += 1
      Thread.stop if max_thread_count && thread_count >= max_thread_count
      Thread.critical = false
    end
    loop do
      begin
        Thread.critical = true
        break if thread_count == 0
        Thread.stop
      ensure
        Thread.critical = false
      end        
    end
    nil
  end
end

module Utility
  def salt
    [[rand(65536)].pack('S')].pack('m').tr('+', '.')
  end

  def exclusive_open(filename, mode = 'r', interval = 5, retrys = 6)
    retrys.times do
      File.open(filename, mode) do |file|
	if file.flock(File::LOCK_EX | File::LOCK_NB) == 0
	  begin
	    return yield(file)
	  ensure
	    file.flock(File::LOCK_UN)
	  end
	end
      end
      sleep(interval)
    end
    nil
  end

  def create_dir(path, public = false)
    return if path.empty? || File.exist?(path)
    create_dir(File.dirname(path), public)
    Dir.mkdir(File.basename(path), public ? 0777 : 0755)
  end    

  def touch(path, public = true)
    create_dir(File.dirname(path), public)
    return if File.basename(path).empty?
    file = File.open(path, 'a')
    file.chmod(public ? 0666 : 0644)
    file.close
  end

  def log(*src)
    return unless Setting::LOG_PATH
    open(Setting::LOG_PATH, 'a') do |file|
      src.each do |line|
        file.puts "#{Time.now}: #{line.chomp}"
        STDOUT.puts line.chomp if $verbose
       end
    end
  end

  def parse_url(url)
    matched = %r|http://(.+?)(/.*)|.match(url)
    return nil unless matched
    host, path = matched[1], matched[2]
    matched = %r|(.+?):(\d+)|.match(host)
    host, port =
      if matched 
	[ matched[1], matched[2].to_i ]
      else 
	[ host, 80 ]
      end
    [ host, port, path ]
  end

  def import(filename)
    matched = %r|(.*?)://(.*)|.match(filename)
    filename, extension = 
      if matched
        [ matched[2], matched[1] ]
      else
        [ filename, filename.dup ]
      end

    compressor = Plugin.compressors.
      find{|plugin| /\.#{plugin.extension}$/.match(extension)}
    if compressor
      extension.sub!(/.#{compressor.extension}/, '')
    end

    translator = Plugin.translators.
      find{|plugin| /#{plugin.extension}$/.match(extension)}

    unless translator
      log "cannot handle #{filename}"
      return []
    end

    data = open(filename){|file| file.read}
    data = compressor.expand(data) if compressor
    translator.import(data)
  end

  def export(filename, sites)
    matched = %r|(.*?)://(.*)|.match(filename)
    filename, extension = 
      if matched
        [ matched[2], matched[1] ]
      else
        [ filename, filename.dup ]
      end

    compressor = Plugin.compressors.
      find{|plugin| /\.#{plugin.extension}$/.match(extension)}
    if compressor
      extension.sub!(/.#{compressor.extension}/, '')
    end

    translator = Plugin.translators.
      find{|plugin| /#{plugin.extension}$/.match(extension)}

    unless translator
      log "cannot handle #{filename}"
      return
    end

    data = ""
    translator.export(data, sites)
    data = compressor.compress(data) if compressor
    open(filename, 'w') do |file| file.write data end
  end

  module_function :salt, :exclusive_open, :create_dir, :touch, :parse_url, 
    :import, :export, :log
end






